@@ -2,6 +2,10 @@
 
 module Pkg
 
+if isdefined(Base, :Experimental) && isdefined(Base.Experimental, Symbol("@max_methods"))
+    @eval Base.Experimental.@max_methods 1
+end
+
 import Random
 import REPL
 import TOML
@@ -11,7 +15,7 @@ export @pkg_str
 export PackageSpec
 export PackageMode, PKGMODE_MANIFEST, PKGMODE_PROJECT
 export UpgradeLevel, UPLEVEL_MAJOR, UPLEVEL_MAJOR, UPLEVEL_MINOR, UPLEVEL_PATCH
-export PreserveLevel, PRESERVE_TIERED, PRESERVE_ALL, PRESERVE_DIRECT, PRESERVE_SEMVER, PRESERVE_NONE
+export PreserveLevel, PRESERVE_TIERED_INSTALLED, PRESERVE_TIERED, PRESERVE_ALL_INSTALLED, PRESERVE_ALL, PRESERVE_DIRECT, PRESERVE_SEMVER, PRESERVE_NONE
 export Registry, RegistrySpec
 
 depots() = Base.DEPOT_PATH
@@ -41,6 +45,7 @@ stdout_f() = something(DEFAULT_IO[], stdout)
 const PREV_ENV_PATH = Ref{String}("")
 
 can_fancyprint(io::IO) = (io isa Base.TTY) && (get(ENV, "CI", nothing) != "true")
+should_autoprecompile() = Base.JLOptions().use_compiled_modules == 1 && get_bool_env("JULIA_PKG_PRECOMPILE_AUTO"; default="true")
 
 include("../ext/LazilyInitializedFields/LazilyInitializedFields.jl")
 
@@ -61,7 +66,7 @@ include("REPLMode/REPLMode.jl")
 import .REPLMode: @pkg_str
 import .Types: UPLEVEL_MAJOR, UPLEVEL_MINOR, UPLEVEL_PATCH, UPLEVEL_FIXED
 import .Types: PKGMODE_MANIFEST, PKGMODE_PROJECT
-import .Types: PRESERVE_TIERED, PRESERVE_ALL, PRESERVE_DIRECT, PRESERVE_SEMVER, PRESERVE_NONE
+import .Types: PRESERVE_TIERED_INSTALLED, PRESERVE_TIERED, PRESERVE_ALL_INSTALLED, PRESERVE_ALL, PRESERVE_DIRECT, PRESERVE_SEMVER, PRESERVE_NONE
 
 # Import artifacts API
 using .Artifacts, .PlatformEngines
@@ -100,8 +105,8 @@ const PreserveLevel = Types.PreserveLevel
 
 # Define new variables so tab comleting Pkg. works.
 """
-    Pkg.add(pkg::Union{String, Vector{String}}; preserve=PRESERVE_TIERED)
-    Pkg.add(pkg::Union{PackageSpec, Vector{PackageSpec}}; preserve=PRESERVE_TIERED)
+    Pkg.add(pkg::Union{String, Vector{String}}; preserve=PRESERVE_TIERED, installed=false)
+    Pkg.add(pkg::Union{PackageSpec, Vector{PackageSpec}}; preserve=PRESERVE_TIERED, installed=false)
 
 Add a package to the current project. This package will be available by using the
 `import` and `using` keywords in the Julia REPL, and if the current project is
@@ -112,18 +117,35 @@ a package, also inside that package.
 The `preserve` keyword argument allows you to key into a specific tier in the resolve algorithm.
 The following table describes the argument values for `preserve` (in order of strictness):
 
-| Value             | Description                                                                         |
-|:------------------|:------------------------------------------------------------------------------------|
-| `PRESERVE_ALL`    | Preserve the state of all existing dependencies (including recursive dependencies)  |
-| `PRESERVE_DIRECT` | Preserve the state of all existing direct dependencies                              |
-| `PRESERVE_SEMVER` | Preserve semver-compatible versions of direct dependencies                          |
-| `PRESERVE_NONE`   | Do not attempt to preserve any version information                                  |
-| `PRESERVE_TIERED` | Use the tier which will preserve the most version information (this is the default) |
+| Value                       | Description                                                                        |
+|:----------------------------|:-----------------------------------------------------------------------------------|
+| `PRESERVE_ALL_INSTALLED`    | Like `PRESERVE_ALL` and only add those already installed                           |
+| `PRESERVE_ALL`              | Preserve the state of all existing dependencies (including recursive dependencies) |
+| `PRESERVE_DIRECT`           | Preserve the state of all existing direct dependencies                             |
+| `PRESERVE_SEMVER`           | Preserve semver-compatible versions of direct dependencies                         |
+| `PRESERVE_NONE`             | Do not attempt to preserve any version information                                 |
+| `PRESERVE_TIERED_INSTALLED` | Like `PRESERVE_TIERED` except `PRESERVE_ALL_INSTALLED` is tried first              |
+| `PRESERVE_TIERED`           | Use the tier that will preserve the most version information while                 |
+|                             | allowing version resolution to succeed (this is the default)                       |
+
+!!! note
+    To change the default strategy to `PRESERVE_TIERED_INSTALLED` set the env var `JULIA_PKG_PRESERVE_TIERED_INSTALLED`
+    to true.
+
+After the installation of new packages the project will be precompiled. For more information see `pkg> ?precompile`.
+
+With the `PRESERVE_ALL_INSTALLED` strategy the newly added packages will likely already be precompiled, but if not this
+may be because either the combination of package versions resolved in this environment has not been resolved and
+precompiled before, or the precompile cache has been deleted by the LRU cache storage
+(see `JULIA_MAX_NUM_PRECOMPILE_FILES`).
+
+!!! compat "Julia 1.9"
+    The `PRESERVE_TIERED_INSTALLED` and `PRESERVE_ALL_INSTALLED` strategies requires at least Julia 1.9.
 
 # Examples
 ```julia
 Pkg.add("Example") # Add a package from registry
-Pkg.add("Example"; preserve=Pkg.PRESERVE_ALL) # Add the `Example` package and preserve existing dependencies
+Pkg.add("Example"; preserve=Pkg.PRESERVE_ALL) # Add the `Example` package and strictly preserve existing dependencies
 Pkg.add(name="Example", version="0.3") # Specify version; latest release in the 0.3 series
 Pkg.add(name="Example", version="0.3.1") # Specify version; exact release
 Pkg.add(url="https://github.com/JuliaLang/Example.jl", rev="master") # From url to remote gitrepo
@@ -131,18 +153,21 @@ Pkg.add(url="/remote/mycompany/juliapackages/OurPackage") # From path to local g
 Pkg.add(url="https://github.com/Company/MonoRepo", subdir="juliapkgs/Package.jl)") # With subdir
 ```
 
-After the installation of new packages the project will be precompiled. See more at [Project Precompilation](@ref).
+After the installation of new packages the project will be precompiled. See more at [Environment Precompilation](@ref).
 
 See also [`PackageSpec`](@ref), [`Pkg.develop`](@ref).
 """
 const add = API.add
 
 """
-    Pkg.precompile(; strict::Bool=false)
-    Pkg.precompile(pkg; strict::Bool=false)
-    Pkg.precompile(pkgs; strict::Bool=false)
+    Pkg.precompile(; strict::Bool=false, timing::Bool=false)
+    Pkg.precompile(pkg; strict::Bool=false, timing::Bool=false)
+    Pkg.precompile(pkgs; strict::Bool=false, timing::Bool=false)
 
 Precompile all or specific dependencies of the project in parallel.
+
+Set `timing=true` to show the duration of the precompilation of each dependency.
+
 !!! note
     Errors will only throw when precompiling the top-level dependencies, given that
     not all manifest dependencies may be loaded by the top-level dependencies on the given system.
@@ -154,13 +179,12 @@ Precompile all or specific dependencies of the project in parallel.
     until they have changed. To disable automatic precompilation set `ENV["JULIA_PKG_PRECOMPILE_AUTO"]=0`.
     To manually control the number of tasks used set `ENV["JULIA_NUM_PRECOMPILE_TASKS"]`.
 
-!!! compat "Julia 1.3"
-    This function requires at least Julia 1.3. On earlier versions
-    you can use `Pkg.API.precompile()` or the `precompile` Pkg REPL command.
-
 !!! compat "Julia 1.8"
     Specifying packages to precompile requires at least Julia 1.8.
 
+!!! compat "Julia 1.9"
+    Timing mode requires at least Julia 1.9.
+
 # Examples
 ```julia
 Pkg.precompile()
@@ -183,14 +207,32 @@ See also [`PackageSpec`](@ref), [`PackageMode`](@ref).
 const rm = API.rm
 
 """
-    Pkg.update(; level::UpgradeLevel=UPLEVEL_MAJOR, mode::PackageMode = PKGMODE_PROJECT)
+    Pkg.why(pkg::Union{String, Vector{String}})
+    Pkg.why(pkg::Union{PackageSpec, Vector{PackageSpec}})
+
+Show the reason why this package is in the manifest.
+The output is a the different way to reach the package
+through the dependency graph starting from the dependencies.
+
+!!! compat "Julia 1.9"
+    This function requires at least Julia 1.9.
+"""
+const why = API.why
+
+"""
+    Pkg.update(; level::UpgradeLevel=UPLEVEL_MAJOR, mode::PackageMode = PKGMODE_PROJECT, preserve::PreserveLevel)
     Pkg.update(pkg::Union{String, Vector{String}})
     Pkg.update(pkg::Union{PackageSpec, Vector{PackageSpec}})
 
-Update a package `pkg`. If no posistional argument is given, update all packages in the manifest if `mode` is `PKGMODE_MANIFEST` and packages in both manifest and project if `mode` is `PKGMODE_PROJECT`.
+If no positional argument is given, update all packages in the manifest if `mode` is `PKGMODE_MANIFEST` and packages in both manifest and project if `mode` is `PKGMODE_PROJECT`.
 If no positional argument is given, `level` can be used to control by how much packages are allowed to be upgraded (major, minor, patch, fixed).
 
-After any package updates the project will be precompiled. See more at [Project Precompilation](@ref).
+If packages are given as positional arguments, the `preserve` argument can be used to control what other packages are allowed to update:
+- `PRESERVE_ALL` (default): Only allow `pkg` to update.
+- `PRESERVE_DIRECT`: Only allow `pkg` and indirect dependencies that are not a direct dependency in the project to update.
+- `PRESERVE_NONE`: Allow `pkg` and all its indirect dependencies to update.
+
+After any package updates the project will be precompiled. See more at [Environment Precompilation](@ref).
 
 See also [`PackageSpec`](@ref), [`PackageMode`](@ref), [`UpgradeLevel`](@ref).
 """
@@ -203,19 +245,21 @@ const update = API.up
 
 **Keyword arguments:**
   - `coverage::Bool=false`: enable or disable generation of coverage statistics.
+  - `allow_reresolve::Bool=true`: allow Pkg to reresolve the package versions in the test environment
   - `julia_args::Union{Cmd, Vector{String}}`: options to be passed the test process.
   - `test_args::Union{Cmd, Vector{String}}`: test arguments (`ARGS`) available in the test process.
 
-!!! compat "Julia 1.3"
-    `julia_args` and `test_args` requires at least Julia 1.3.
+!!! compat "Julia 1.9"
+    `allow_reresolve` requires at least Julia 1.9.
 
 Run the tests for package `pkg`, or for the current project (which thus needs to be a package) if no
 positional argument is given to `Pkg.test`. A package is tested by running its
 `test/runtests.jl` file.
 
-The tests are run by generating a temporary environment with only `pkg` and its (recursive) dependencies
-in it. If a manifest exists, the versions in that manifest are used, otherwise
-a feasible set of packages is resolved and installed.
+The tests are run by generating a temporary environment with only the `pkg` package
+and its (recursive) dependencies in it. If a manifest file exists and the `allow_reresolve`
+keyword argument is set to `false`, the versions in the manifest file are used.
+Otherwise a feasible set of packages is resolved and installed.
 
 During the tests, test-specific dependencies are active, which are
 given in the project file as e.g.
@@ -231,7 +275,25 @@ test = ["Test"]
 The tests are executed in a new process with `check-bounds=yes` and by default `startup-file=no`.
 If using the startup file (`~/.julia/config/startup.jl`) is desired, start julia with `--startup-file=yes`.
 Inlining of functions during testing can be disabled (for better coverage accuracy)
-by starting julia with `--inline=no`.
+by starting julia with `--inline=no`. The tests can be run as if different command line arguments were
+passed to julia by passing the arguments instead to the `julia_args` keyword argument, e.g.
+
+```
+Pkg.test("foo"; julia_args=["--inline"])
+```
+
+To pass some command line arguments to be used in the tests themselves, pass the arguments to the
+`test_args` keyword argument. These could be used to control the code being tested, or to control the
+tests in some way. For example, the tests could have optional additional tests:
+```
+if "--extended" in ARGS
+    @test some_function()
+end
+```
+which could be enabled by testing with
+```
+Pkg.test("foo"; test_args=["--extended"])
+```
 """
 const test = API.test
 
@@ -265,39 +327,51 @@ redirecting to the `build.log` file.
 const build = API.build
 
 """
-    Pkg.pin(pkg::Union{String, Vector{String}}; io::IO=stderr)
-    Pkg.pin(pkgs::Union{PackageSpec, Vector{PackageSpec}}; io::IO=stderr)
+    Pkg.pin(pkg::Union{String, Vector{String}}; io::IO=stderr, all_pkgs::Bool=false)
+    Pkg.pin(pkgs::Union{PackageSpec, Vector{PackageSpec}}; io::IO=stderr, all_pkgs::Bool=false)
 
 Pin a package to the current version (or the one given in the `PackageSpec`) or to a certain
-git revision. A pinned package is never updated.
+git revision. A pinned package is never automatically updated: if `pkg` is tracking a path,
+or a repository, those remain tracked but will not update.
+To get updates from the origin path or remote repository the package must first be freed.
+
+!!! compat "Julia 1.7"
+    The `all_pkgs` kwarg was introduced in julia 1.7.
 
 # Examples
 ```julia
 Pkg.pin("Example")
 Pkg.pin(name="Example", version="0.3.1")
+Pkg.pin(all_pkgs = true)
 ```
 """
 const pin = API.pin
 
 """
-    Pkg.free(pkg::Union{String, Vector{String}}; io::IO=stderr)
-    Pkg.free(pkgs::Union{PackageSpec, Vector{PackageSpec}}; io::IO=stderr)
+    Pkg.free(pkg::Union{String, Vector{String}}; io::IO=stderr, all_pkgs::Bool=false)
+    Pkg.free(pkgs::Union{PackageSpec, Vector{PackageSpec}}; io::IO=stderr, all_pkgs::Bool=false)
 
 If `pkg` is pinned, remove the pin.
-If `pkg` is tracking a path,
-e.g. after [`Pkg.develop`](@ref), go back to tracking registered versions.
+If `pkg` is tracking a path, e.g. after [`Pkg.develop`](@ref), go back to tracking registered versions.
+To free all dependencies set `all_pkgs=true`.
+
+!!! compat "Julia 1.7"
+    The `all_pkgs` kwarg was introduced in julia 1.7.
 
 # Examples
 ```julia
 Pkg.free("Package")
+Pkg.free(all_pkgs = true)
 ```
+
+
 """
 const free = API.free
 
 
 """
-    Pkg.develop(pkg::Union{String, Vector{String}}; io::IO=stderr)
-    Pkg.develop(pkgs::Union{Packagespec, Vector{Packagespec}}; io::IO=stderr)
+    Pkg.develop(pkg::Union{String, Vector{String}}; io::IO=stderr, preserve=PRESERVE_TIERED, installed=false)
+    Pkg.develop(pkgs::Union{PackageSpec, Vector{PackageSpec}}; io::IO=stderr, preserve=PRESERVE_TIERED, installed=false)
 
 Make a package available for development by tracking it by path.
 If `pkg` is given with only a name or by a URL, the package will be downloaded
@@ -306,6 +380,9 @@ to the location specified by the environment variable `JULIA_PKG_DEVDIR`, with
 
 If `pkg` is given as a local path, the package at that path will be tracked.
 
+The preserve strategies offered by `Pkg.add` are also available via the `preserve` kwarg.
+See [`Pkg.add`](@ref) for more information.
+
 # Examples
 ```julia
 # By name
@@ -333,31 +410,34 @@ const generate = API.generate
 """
     Pkg.dependencies()::Dict{UUID, PackageInfo}
 
-!!! compat "Julia 1.4"
-    This feature requires Julia 1.4, and is considered experimental.
+This feature is considered experimental.
 
-Query the dependency graph.
+Query the dependency graph of the active project.
 The result is a `Dict` that maps a package UUID to a `PackageInfo` struct representing the dependency (a package).
 
 # `PackageInfo` fields
 
-| Field             | Description                                                |
-|:------------------|:-----------------------------------------------------------|
-| `name`            | The name of the package                                    |
-| `version`         | The version of the package (this is `Nothing` for stdlibs) |
-| `is_direct_dep`   | The package is a direct dependency                         |
-| `is_tracking_path`| Whether a package is directly tracking a directory         |
-| `is_pinned`       | Whether a package is pinned                                |
-| `source`          | The directory containing the source code for that package  |
-| `dependencies`    | The dependencies of that package as a vector of UUIDs      |
+| Field                 | Description                                                                       |
+|:------------------    |:----------------------------------------------------------------------------------|
+| `name`                | The name of the package                                                           |
+| `version`             | The version of the package (this is `Nothing` for stdlibs)                        |
+| `tree_hash`           | A file hash of the package directory tree                                         |
+| `is_direct_dep`       | The package is a direct dependency                                                |
+| `is_pinned`           | Whether a package is pinned                                                       |
+| `is_tracking_path`    | Whether a package is tracking a path                                              |
+| `is_tracking_repo`    | Whether a package is tracking a repository                                        |
+| `is_tracking_registry`| Whether a package is being tracked by registry i.e. not by path nor by repository |
+| `git_revision`        | The git revision when tracking by repository                                      |
+| `git_source`          | The git source when tracking by repository                                        |
+| `source`              | The directory containing the source code for that package                         |
+| `dependencies`        | The dependencies of that package as a vector of UUIDs                             |
 """
 const dependencies = API.dependencies
 
 """
     Pkg.project()::ProjectInfo
 
-!!! compat "Julia 1.4"
-    This feature requires Julia 1.4, and is considered experimental.
+This feature is considered experimental.
 
 Request a `ProjectInfo` struct which contains information about the active project.
 
@@ -368,6 +448,7 @@ Request a `ProjectInfo` struct which contains information about the active proje
 | name         | The project's name                                                                          |
 | uuid         | The project's UUID                                                                          |
 | version      | The project's version                                                                       |
+| ispackage    | Whether the project is a package (has a name and uuid)                                      |
 | dependencies | The project's direct dependencies as a `Dict` which maps dependency name to dependency UUID |
 | path         | The location of the project file which defines the active project                           |
 """
@@ -386,7 +467,7 @@ If no `Project.toml` exist in the current active project, create one with all th
 dependencies in the manifest and instantiate the resulting project.
 
 After packages have been installed the project will be precompiled.
-See more at [Project Precompilation](@ref).
+See more at [Environment Precompilation](@ref).
 """
 const instantiate = API.instantiate
 
@@ -399,12 +480,14 @@ from packages that are tracking a path.
 const resolve = API.resolve
 
 """
-    Pkg.status([pkgs...]; mode::PackageMode=PKGMODE_PROJECT, diff::Bool=false, compat::Bool=false, io::IO=stdout)
+    Pkg.status([pkgs...]; outdated::Bool=false, mode::PackageMode=PKGMODE_PROJECT, diff::Bool=false, compat::Bool=false, extensions::Bool=false, io::IO=stdout)
+
 
 Print out the status of the project/manifest.
 
 Packages marked with `⌃` have new versions that can be installed, e.g. via [`Pkg.up`](@ref).
-Those marked with `⌅` have new versions available, but that cannot be installed. To see why use the `outdated` kwarg.
+Those marked with `⌅` have new versions available, but cannot be installed due to compatibility conflicts with other packages. To see why, set the
+keyword argument `outdated=true`.
 
 Setting `outdated=true` will only show packages that are not on the latest version,
 their maximum version and why they are not on the latest version (either due to other
@@ -429,19 +512,15 @@ that are in the project (explicitly added). If `mode` is `PKGMODE_MANIFEST`,
 print status also about those in the manifest (recursive dependencies). If there are
 any packages listed as arguments, the output will be limited to those packages.
 
+Setting `ext=true` will show dependencies with extensions and what extension dependencies
+of those that are currently loaded.
+
 Setting `diff=true` will, if the environment is in a git repository, limit
 the output to the difference as compared to the last git commit.
 
 See [`Pkg.project`](@ref) and [`Pkg.dependencies`](@ref) to get the project/manifest
 status as a Julia object instead of printing it.
 
-!!! compat "Julia 1.1"
-    `Pkg.status` with package arguments requires at least Julia 1.1.
-
-!!! compat "Julia 1.3"
-    The `diff` keyword argument requires at least Julia 1.3. In earlier versions `diff=true`
-    is the default for environments in git repositories.
-
 !!! compat "Julia 1.8"
     The `⌃` and `⌅` indicators were added in Julia 1.8.
     The `outdated` keyword argument requires at least Julia 1.8.
@@ -480,9 +559,7 @@ The logic for what path is activated is as follows:
     activate the environment at the tracked path.
   * Otherwise, `s` is interpreted as a non-existing path, which is then activated.
 
-If no argument is given to `activate`, then activate the home project.
-The home project is specified by either the `--project` command line option to
-the julia executable, or the `JULIA_PROJECT` environment variable.
+If no argument is given to `activate`, then use the first project found in `LOAD_PATH`.
 
 # Examples
 ```
@@ -491,9 +568,6 @@ Pkg.activate("local/path")
 Pkg.activate("MyDependency")
 Pkg.activate(; temp=true)
 ```
-
-!!! compat "Julia 1.4"
-    the `temp` option requires at least Julia 1.4.
 """
 const activate = API.activate
 
@@ -508,9 +582,6 @@ versions that are already downloaded in version resolution.
 
 To work in offline mode across Julia sessions you can
 set the environment variable `JULIA_PKG_OFFLINE` to `"true"`.
-
-!!! compat "Julia 1.5"
-    Pkg's offline mode requires Julia 1.5 or later.
 """
 offline(b::Bool=true) = (OFFLINE_MODE[] = b; nothing)
 
@@ -537,7 +608,9 @@ This includes:
   * The `name` of the package.
   * The package's unique `uuid`.
   * A `version` (for example when adding a package). When upgrading, can also be an instance of
-    the enum [`UpgradeLevel`](@ref).
+    the enum [`UpgradeLevel`](@ref). If the version is given as a `String` this means that unspecified versions
+    are "free", for example `version="0.5"` allows any version `0.5.x` to be installed. If given as a `VersionNumber`,
+    the exact version is used, for example `version=v"0.5.3"`.
   * A `url` and an optional git `rev`ision. `rev` can be a branch name or a git commit SHA1.
   * A local `path`. This is equivalent to using the `url` argument but can be more descriptive.
   * A `subdir` which can be used when adding a package that is not in the root of a repository.
@@ -545,15 +618,14 @@ This includes:
 Most functions in Pkg take a `Vector` of `PackageSpec` and do the operation on all the packages
 in the vector.
 
-!!! compat "Julia 1.5"
-    Many functions that take a `PackageSpec` or a `Vector{PackageSpec}` can be called with a more concise notation with `NamedTuple`s.
-    For example, `Pkg.add` can be called either as the explicit or concise versions as:
+Many functions that take a `PackageSpec` or a `Vector{PackageSpec}` can be called with a more concise notation with `NamedTuple`s.
+For example, `Pkg.add` can be called either as the explicit or concise versions as:
 
-    | Explicit                                                            | Concise                                        |
-    |:--------------------------------------------------------------------|:-----------------------------------------------|
-    | `Pkg.add(PackageSpec(name="Package"))`                              | `Pkg.add(name = "Package")`                    |
-    | `Pkg.add(PackageSpec(url="www.myhost.com/MyPkg")))`                 | `Pkg.add(name = "Package")`                    |
-    |` Pkg.add([PackageSpec(name="Package"), PackageSpec(path="/MyPkg"])` | `Pkg.add([(;name="Package"), (;path="MyPkg")])`|
+| Explicit                                                            | Concise                                        |
+|:--------------------------------------------------------------------|:-----------------------------------------------|
+| `Pkg.add(PackageSpec(name="Package"))`                              | `Pkg.add(name = "Package")`                    |
+| `Pkg.add(PackageSpec(url="www.myhost.com/MyPkg")))`                 | `Pkg.add(name = "Package")`                    |
+|` Pkg.add([PackageSpec(name="Package"), PackageSpec(path="/MyPkg"])` | `Pkg.add([(;name="Package"), (;path="MyPkg")])`|
 
 Below is a comparison between the REPL mode and the functional API:
 
@@ -561,11 +633,12 @@ Below is a comparison between the REPL mode and the functional API:
 |:---------------------|:------------------------------------------------------|
 | `Package`            | `PackageSpec("Package")`                              |
 | `Package@0.2`        | `PackageSpec(name="Package", version="0.2")`          |
+| -                    | `PackageSpec(name="Package", version=v"0.2.1")`       |
 | `Package=a67d...`    | `PackageSpec(name="Package", uuid="a67d...")`         |
 | `Package#master`     | `PackageSpec(name="Package", rev="master")`           |
 | `local/path#feature` | `PackageSpec(path="local/path"; rev="feature")`       |
 | `www.mypkg.com`      | `PackageSpec(url="www.mypkg.com")`                    |
-| `--major Package`    | `PackageSpec(name="Package", version=PKGLEVEL_MAJOR)` |
+| `--major Package`    | `PackageSpec(name="Package", version=UPLEVEL_MAJOR)` |
 
 """
 const PackageSpec = Types.PackageSpec
@@ -616,19 +689,16 @@ A `RegistrySpec` is a representation of a registry with various metadata, much l
 Most registry functions in Pkg take a `Vector` of `RegistrySpec` and do the operation
 on all the registries in the vector.
 
-!!! compat "Julia 1.1"
-    Pkg's registry handling requires at least Julia 1.1.
-
 # Examples
 
 Below is a comparison between the REPL mode and the functional API::
 
-| `REPL`               | `API`                                           |
-|:---------------------|:------------------------------------------------|
-| `Registry`           | `RegistrySpec("Registry")`                      |
-| `Registry=a67d...`   | `RegistrySpec(name="Registry", uuid="a67d...")` |
-| `local/path`         | `RegistrySpec(path="local/path")`               |
-| `www.myregistry.com` | `RegistrySpec(url="www.myregistry.com")`        |
+| `REPL`               | `API`                                             |
+|:---------------------|:--------------------------------------------------|
+| `MyRegistry`         | `RegistrySpec("MyRegistry")`                      |
+| `MyRegistry=a67d...` | `RegistrySpec(name="MyRegistry", uuid="a67d...")` |
+| `local/path`         | `RegistrySpec(path="local/path")`                 |
+| `www.myregistry.com` | `RegistrySpec(url="www.myregistry.com")`          |
 """
 const RegistrySpec = Registry.RegistrySpec
 
@@ -731,9 +801,9 @@ end
 # Precompilation #
 ##################
 
-function _auto_precompile(ctx::Types.Context; warn_loaded = true, already_instantiated = false)
-    if Base.JLOptions().use_compiled_modules == 1 && get_bool_env("JULIA_PKG_PRECOMPILE_AUTO"; default="true")
-        Pkg.precompile(ctx; internal_call=true, warn_loaded = warn_loaded, already_instantiated = already_instantiated)
+function _auto_precompile(ctx::Types.Context, pkgs::Vector{PackageSpec}=PackageSpec[]; warn_loaded = true, already_instantiated = false)
+    if should_autoprecompile()
+        Pkg.precompile(ctx, pkgs; internal_call=true, warn_loaded = warn_loaded, already_instantiated = already_instantiated)
     end
 end
 
